/*
 * this file is a part of yabbs - yet another bulletin board system.
 * Copyright (C) 1993 Alex Wetmore.  
 * email: aw2t@andrew.cmu.edu
 * address: 6 rech ave
 *          oreland pa 19075
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "bbsdefs.h"                        /* function prototypes      */

/*
 * the configuration menu.  Just call other procedures as needed.
 */

int doconfig(void) {
    char    cmd, temp[7];
    int     bye, done;

    bye = done = FALSE;
    while (!done) {
        clrscr(); drawuser();
        if (!(userinfo.options & O_XPERT)) sputs_mime(MNU_CONFIG);
        sputs("config: [plsxiqg?] ");
        cmd = waitfor("plsxiqg?");
        switch (lower(cmd)) {
            case '?' :  
                sputs("help\n\n"); 
                sputs_mime(MNU_CONFIG); 
                waitkey(); 
                break;
            case 'p' :  
                sputs("password\n\n"); 
                changepw(); 
                break;
            case 'l' : 
                sputs("lines\n\nIf you set lines to 0 \"more prompting\" will be disabled.\n");
                sputs("Also, setting this to 10 or less is not recommended.\n\n");
                sputs("How many lines does your screen have? ");
                sgets(temp, 5);
                userinfo.scrlen = atoi(temp);
                break;
            case 's' : 
                sputs("subscriptions\n\n"); 
                changesubs();
                break;
            case 'x' : 
                sputs("expert toggle\n\n"); 
                userinfo.options = userinfo.options ^ O_XPERT; 
                break;
            case 'i' :
                sputs("info file edit\n\n"); 
                changeplan(userinfo.name); 
                break;
            case 'g' : 
                sputs("goodbye\n\n"); 
                done = bye = TRUE; 
                break;
            case 'q' : 
                sputs("quit\n\n"); 
                done = TRUE; 
                bye = FALSE; 
                break;
            default  : 
                sputs("what?\n\n");
                break;
        }
    }
    saveopts();
    return bye;
}

/*
 * make a new user.  Ask them for their name, password, and then run
 * doconfig to let them do the rest.  They are automatically given access
 * to email and the first sub, and subscribed to both.
 */

int newuser(void) {
    char    name[9];
    int     okayname = FALSE;
        
    dumpsvr(F_NEWUSER);
    do {
        sputs("What would you like your name to be? (8 chars) ");
        sgets(name, 8);
        if (bbs_newuser(name, "newuser")) okayname = TRUE;
    } while (!okayname);
    strcpy(userinfo.name, name);
    changepw();
    return 0;
}

/*
 * allow a user to login.  If they enter new, create a new user, otherwise
 * check to make sure that their password and id match.  If they can't get
 * in after 5 trys dump them, otherwise return their user number.
 */

int login(void) {
    char    name[9], pass[9], buf[255];
    int     usr, attempts = 0, okaylogin;

    do {
        sputs("\nlogin: "); sgets(name, 8);
        if (strcmp(name, "new") == 0) {
            usr = newuser();
            strcpy(name, userinfo.name);
            okaylogin = TRUE;
        } else {
            sputs("password: "); sgetpw(pass);
            okaylogin = bbs_login(name, pass);
        }
    } while ((!okaylogin) && (++attempts < 10));
    strcpy(userinfo.name, name);
    if (attempts >= 10) {
        sputs("\nYou appear to being having trouble accessing your account,\n");
        sputs("please try again later.\n");
        return FALSE;
    }
    bbs_getoptions();
    clrscr();
    sprintf(buf, "welcome to yabbs, %s\n", name); sputs(buf);
    dumpsvr(F_NEWS);
    return TRUE;
}

/*
 * get the terminal type and other information.
 */

void getterm(void) {
    char buf[80], **area;
    int  i;

    strcpy(termname, getenv("TERM"));
    sputs("If you don't know your terminal type, enter \"none\" at the next prompt.\n");
    sputs("Hit enter to accept the default type (in []'s).\n");
    i = FALSE;
    sprintf(buf, "What is your terminal type? [%s] ", termname); sputs(buf);
    sgets(buf, 32);
    
    if ((strcmp(buf, "none") == 0) || 
          (strcmp(buf, "unknown") == 0) || 
          (strcmp(buf, "dumb") == 0)) {
        strcpy(termname, "none");
    } else if (buf[0] != 0) {
        strcpy(termname, buf);
    }
    if (strcmp(termname, "none") == 0) {
        sputs("\nUsing dumb terminal\n\n\n\n\n\n"); 
        strcpy(t_cleol, "                                                                              \r");
    } else if (tgetent(termcap, termname) != 1) {
        sputs("\nThere is no termcap entry for you terminal, using no terminal features\n\n\n\n\n\n\n");
        strcpy(t_cleol, "                                                                              \r");
        strcpy(termname, "none");
    } else {
        area = malloc(sizeof(area));
        area[0] = malloc(sizeof(char) * 1024);
        strcpy(t_clrscr, tgetstr("cl", area));
        if (tgetstr("cl", area) != NULL) strcpy(t_clrscr, tgetstr("cl", area));
        else *t_clrscr = 0;
        if (tgetstr("md", area) != NULL) {
            strcpy(t_hion, tgetstr("md", area));
            strcpy(t_hioff, tgetstr("me", area));
        } else *t_hioff = *t_hion = 0;
        if (tgetstr("ce", area) != NULL) strcpy(t_cleol, tgetstr("ce", area));
        else *t_cleol = 0;
        if (tgetstr("up", area) != NULL) strcpy(t_upcur, tgetstr("up", area));
        else *t_upcur = 0;
    }
    t_puts(t_cleol);
    writec('\n');
    clrscr();
}

/*
 * the main menu of the bbs.  Just call the appropriate procedure...
 */

void dobbs(void) {
    char    cmd, keystr[20], st[80];
    int     done;

    readbases();
    drawuser();
    done = FALSE;
    while (!done) {
        sputs("\n");
        if (!(userinfo.options & O_XPERT)) sputs_mime(MNU_MAIN);
        strcpy(keystr, "msciwtyfg!?"); 
        sprintf(st, "main: [%s] ", keystr);
        sputs(st);
        cmd = waitfor(keystr);
        sprintf(st, "%c) ", lower(cmd)); sputs(st);
        switch (lower(cmd)) {
            case '?' : sputs("help\n\n"); sputs_mime(MNU_MAIN); break;
            case 'm' : sputs("messages\n\n"); clrscr(); done = domessages(); 
                       break;
            case 's' : sputs("setup\n\n"); done = doconfig(); break;
            case 'c' : sputs("talk channels\n\n"); getchannels(); break;
            case 'i' : sputs("info on user\n\n"); readplan(userinfo.name); 
                       break;
            case 'w' : sputs("who\n\n"); who(); break;
            case 'g' : sputs("goodbye\n\n"); 
                       done = ynprompt("Are you sure? [Yn] ", TRUE); break;
            case 'y' : sputs("yalk\n\n"); dotalk(); break;
            case 't' : sputs("talk\n\n"); dotalk(); break;
            case 'u' : sputs(
                             "undocumented features, we gotta lot of them\n\n"
                       ); break;
            case 'f' : sputs("gfiles\n\n"); dogfil(""); break;
            case '!' : sputs("gnuinfo\n\n"); dumpsvr("COPYING-2.0"); break;
            default  : sputs("what?\n\n");
        }
    }
    sputs("\nlater . . .");
    sputs("\nPlease send any feedback that you may have about this software to:");
    sputs("\n  htoaster+@cmu.edu\n");
}

/*
 * the main loop.  Wait for a connection, let the person login, if they do
 * succesfully then give them the main menu, otherwise drop their connection.
 */

void yabbsmain(int argc, char **argv) {
    char    timeb[35];
    time_t  tp;
    char    buf[255];
    char    *onfrom;

    signal(SIGINT, sigquit);                /* normal quit signals      */
    signal(SIGQUIT, sigquit);
    signal(SIGTSTP, sigstop);               /* stop signal              */
    signal(SIGCONT, sigcont);               /* stop signal              */

    userinfo.scrlen = 24;                   /* start with a safe scrlen */

    sprintf(buf, "HOME=%s", F_HOME);
    putenv(buf);
    
    sputs("\n");
    sputs(YABBSNAME);
    sputs(" unix client"); sputs(" - version "); 
    sputs(CLIENTVERSION);
    sputs(YABBSGNUHI);
    sputs(YABBSGNUCLI);

    /* connect to a server                                              */
    if (argc == 2) bbs_connect(argv[1]);
    else bbs_connect(SERVER);

    getterm();
    
    dumpsvr(F_WELCOME);                     /* dump welcome file        */

    tp = time(NULL);                        /* get the time into a stng */
    strcpy(timeb, ( char *) ctime(&tp));
    timeb[strlen(timeb) - 1] = 0;
    sputs("the last few users: (the current time is "); 
    sputs(timeb); sputs(")\n");
    dumpsvr(F_LASTU);                       /* dump lastuser file       */

    if ((onfrom = getenv("HOSTFROM")) != NULL) bbs_onfrom(strlwr(onfrom));

    if (login()) dobbs();

    killconnection(K_SILENT);               /* logout                   */
}

/*
 * list all online users from the command line
 */

void yahoomain(int argc, char **argv) {
    who();
}
