/*
 * this file is a part of yabbs - yet another bulletin board system.
 * Copyright (C) 1993 Alex Wetmore.  
 * email: aw2t@andrew.cmu.edu
 * address: 6 rech ave
 *          oreland pa 19075
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * yalk.c - part of the yabbs bulletin board package by alex wetmore
 *          see bbs.c for information on the system and copying.
 */

/*
 * thanks to dmonger, pw1r@andrew.cmu.edu, for the volume hacks
 */

#include "bbsdefs.h"
#include "llist.h"

char curchan[15];                       /* current channel                  */
llist channels;                         /* channels we are on               */
llist ignores;                          /* people we want to ignore         */
llist nocaps;                           /* people we want to attenuate      */
int intalk;

#define isignored(name) inlist(ignores, name, stringptrcomp)
#define isnocaps(name) inlist(nocaps, name, stringptrcomp)

void setcurchan(char *cname) {
    char buf[255];

    strcpy(curchan, cname);
    sprintf(buf, "*** current channel is %s\n", curchan);
    sputs(buf);
}

void getchannels(void) {
    char buf[255];
    
    sprintf(buf, "\016%12s : users\017\n", "name");
    sputs_wr(buf);
    bbs_channels();
}

int stringptrcomp(void *p1, void *p2) {
    return (strcmp(strlwr((char *) p1), strlwr((char *) p2)) == 0);
}

void cli_channelline(char *cname, char *users) {
    char buf[255], newusers[300];
    int i, l, j;
    
    for (i = j = 0, l = strlen(users); i < l; i++) {
        if (users[i] == ',') { newusers[j++] = ','; newusers[j++] = ' '; }
        else newusers[j++] = users[i];
    }
    newusers[j] = 0;
    sprintf(buf, "%12s : %s\n", 
        cname, newusers);
    sputs_process = FALSE; sputs_wr(buf); sputs_process = TRUE;
}

char *lowermsg(char *msg) {
    int i, l;

    for (i = 0, l = strlen(msg); i < l; i++) msg[i] = lower(msg[i]);
    return msg;
}

/*
 * incoming text from server
 */

void cli_talkline(char *channel, char *name, char *msg, int type) {
    char buf[1024];

    strcpy(buf, name);

    /* check to see if we should be ignoring this person                */
    if (isignored(buf)) return;

    /* check to see if we are lowercasing this persons text             */
    if (isnocaps(buf)) lowermsg(msg);

    if (intalk) { sputs("\r"); t_puts(t_cleol); }
    switch (type) {
        case TL_PUBLIC:
            if (strcmp(channel, curchan) == 0) 
                sprintf(buf, "<\016%s\017> %s\n", name, msg); 
            else
                sprintf(buf, "<\016%s:%s\017> %s\n", name, channel, msg); 
            break;
        case TL_PRIVATE:
            sprintf(buf, "*\016%s\017* %s\n", name, msg); 
            break;
        case TL_ACTION:
            if (strcmp(channel, curchan) == 0) 
                sprintf(buf, "\016%s\017 %s\n", name, msg); 
            else
                sprintf(buf, "\016%s:%s\017 %s\n", name, channel, msg); 
            break;
        case TL_HIDE: 
            sprintf(buf, "*** \016%s\017 made hidden by %s\n", channel, name); 
            break;
        case TL_SHOW: 
            sprintf(buf, "*** \016%s\017 made visible by %s\n", channel, name); 
            break;
        case TL_JOIN: 
            sprintf(buf,    
                "*** \016%s\017 has joined channel %s\n", name, channel); 
            break;
        case TL_LEAVE: 
            sprintf(buf, 
                "*** \016%s\017 has left channel %s\n", name, channel); 
            break;
    }
    if (intalk) sputs_wr(buf);
    line = 1;
    redraw = TRUE;
}

void cli_page(char *from) {
    char buf[255];

    if (isignored(from)) return;

    sprintf(buf, "\nPage Request: [%s] would like to talk to you\n\n", from);
    sputs(buf);
    redraw = TRUE;
}

/*
 * allow a user in chat to page someone outside of chat.
 */

void dopage(char *tstr) {
    char    buf[1024], name[255];
    int     i;

    for (i = 0; tstr[i] != ' ' && tstr[i] != 0; i++); 
    if (tstr[i] == ' ') i++; strmid(name, tstr, i, strlen(tstr) - i);
    bbs_page(name);
    sprintf(buf, "*** sent page to \016%s\017\n", name);
    sputs(buf);
}
        
void doinfo(char *tstr) {
    char    buf[1024], name[255];
    int     i;

    for (i = 0; (tstr[i] != ' ') && (tstr[i] != 0); i++); 
    if (tstr[i] == ' ') i++; 
    strmid(name, tstr, i, strlen(tstr) - i);
    if (strlen(name) != 0) {
        if (!finduser(name)) {
            sprintf(buf, "*** no such user %s\n", name);
            sputs(buf);
        } else {
            sprintf(buf, "*** ---- plan file for %s ----\n", name);
            sputs(buf);
            bbs_getplanfile(name);
            sputs("*** ---- end of plan ----\n");
        }
    } else sputs("*** usage: /info <handle>\n");
}

void dojoin(char *tstr) {
    char    name[255], *cname;
    int     i;

    for (i = 0; (tstr[i] != ' ') && (tstr[i] != 0); i++); 
    if (tstr[i] == ' ') i++; 
    strmid(name, tstr, i, strlen(tstr) - i);
    if (strlen(name) != 0) {
        if ((name[0] == '*') && (name[strlen(name) - 1] == '*')) {
            name[strlen(name) - 1] = 0;
            cname = (char *) malloc(sizeof(char) * (strlen(name) + 1));
            strcpy(cname, &name[1]);
            if (!inlist(channels, (void *) cname, stringptrcomp))
                channels = addtolist(channels, (void *) cname);
            setcurchan(cname);
            bbs_talkjoin(cname);
            bbs_talkhide(cname);
        } else {
            cname = (char *) malloc(sizeof(char) * (strlen(name) + 1));
            strcpy(cname, name);
            if (!inlist(channels, (void *) cname, stringptrcomp))
                channels = addtolist(channels, (void *) cname);
            setcurchan(cname);
            bbs_talkjoin(cname);
        }
    } else sputs("*** usage: /join <channel name>\n");
}
        
void doleave(char *tstr) {
    char    name[255];
    int     i;

    for (i = 0; (tstr[i] != ' ') && (tstr[i] != 0); i++); 
    if (tstr[i] == ' ') i++; 
    strmid(name, tstr, i, strlen(tstr) - i);
    if ((name[0] == '*') && (name[strlen(name) - 1] == '*')) {
        strcpy(name, &name[1]);
        name[strlen(name) - 1] = 0;
    }
    if (strlen(name) != 0) {
        bbs_talkleave(name);
        channels = deletefromlist(channels, (void *) name, stringptrcomp);
        if (channels != NULL) setcurchan((char *) channels->ptr);
        else sputs("*** you are in limbo, use /j to join a channel\n");
    } else sputs("*** usage: /leave <channel name>\n");
}
        
/*
 * send a message to all of the users in the talk section
 */

void doaction(char *tmsg) {
    bbs_talkaction(curchan, &(tmsg[1]));
}

/*
 * send a message to all of the users in the talk section
 */

void dowrite(char *tmsg) {
    char buf[1024];

    if (elite) {
        sprintf(buf, "d00d, %s", tmsg);
    } else if (yeck) {
        sprintf(buf, "dude, %s, dude!", tmsg);
    } else {
        sprintf(buf, "%s", tmsg);
    }
    bbs_talkpublic(curchan, buf);
}

/*
 * send a message to one user in the talk section 
 */

void domsg(char *tmsg) {
    int i, j;
    char to[255], buf[255], buf2[255];

    /* find the spaces surrounding the user name and extract it         */
    for (i = 0; tmsg[i] != ' ' && tmsg[i] != 0; i++); 
    if (tmsg[i] == ' ') i++; 
    for (j = i; tmsg[j] != ' ' && tmsg[j] != 0; j++);
    strmid(to, tmsg, i, j - i);
    if (tmsg[j] == ' ') j++;

    if (elite) {
        sprintf(buf, "d00d, %s", &tmsg[j]);
        sprintf(buf2, "=> *\016%s\017* d00d, %s\n", to, &tmsg[j]);
    } else if (yeck) {
        sprintf(buf, "dude, %s, dude!", &tmsg[j]);
        sprintf(buf2, "=> *\016%s\017* dude, %s, dude!\n", to, &tmsg[j]);
    } else {
        sprintf(buf, "%s", &tmsg[j]);
        sprintf(buf2, "=> *\016%s\017* %s\n", to, &tmsg[j]);
    }
    bbs_talkprivate(to, buf);
    sputs_wr(buf2);
}

void doignore(char *tstr) {
    char    buf[1024], name[255], *namep;
    int     i;
    llist   p;

    for (i = 0; (tstr[i] != ' ') && (tstr[i] != 0); i++); 
    if (tstr[i] == ' ') i++; 
    strmid(name, tstr, i, strlen(tstr) - i);
    if (strlen(name) != 0) {
        if (!finduser(name)) {
            sprintf(buf, "*** no such user %s\n", name);
            sputs(buf);
        } else {
            if (isignored(name)) {
                ignores = deletefromlist(ignores, (void *) name, stringptrcomp);
                sprintf(buf, 
                    "*** no longer ignoring <Bold>%s</Bold><nl>", name);
            } else {
                namep = (char *) malloc(sizeof(char) * (strlen(name) + 1));
                strcpy(namep, name);
                ignores = addtolist(ignores, (void *) namep);
                sprintf(buf, "*** ignoring <Bold>%s</Bold><nl>", name);
            }
            sputs_mime(buf);
        }
    } else {
        p = ignores;
        sputs("*** people you're ignoring:\n");
        while (p != NULL) {
            sprintf(buf, "*** <Bold>%s</Bold><nl>", p->ptr);
            sputs_mime(buf);
            p = p->next;
        }
        sputs("*** end of ignore list\n");
    }
}

void donocaps(char *tstr) {
    char    buf[1024], name[255], *namep;
    int     i;
    llist   p;

    for (i = 0; (tstr[i] != ' ') && (tstr[i] != 0); i++); 
    if (tstr[i] == ' ') i++; 
    strmid(name, tstr, i, strlen(tstr) - i);
    if (strlen(name) != 0) {
        if (!finduser(name)) {
            sprintf(buf, "*** no such user %s\n", name);
            sputs(buf);
        } else {
            if (isnocaps(name)) {
                nocaps = deletefromlist(nocaps, (void *) name, stringptrcomp);
                sprintf(buf, 
                    "*** no longer lowercasing text from <Bold>%s</Bold><nl>", 
                name);
            } else {
                namep = (char *) malloc(sizeof(char) * (strlen(name) + 1));
                strcpy(namep, name);
                nocaps = addtolist(nocaps, (void *) namep);
                sprintf(buf, 
                    "*** lowercasing all text from <Bold>%s</Bold><nl>", name);
            }
            sputs_mime(buf);
        }
    } else {
        p = nocaps;
        sputs("*** people you're lowercasing text from:\n");
        while (p != NULL) {
            sprintf(buf, "*** <Bold>%s</Bold><nl>", p->ptr);
            sputs_mime(buf);
            p = p->next;
        }
        sputs("*** end of nocaps list\n");
    }
}

/*
 * talk with other users (just a simple chat section).
 */

void dotalk(void) {
    int     done = FALSE;
    char    tmsg[255], buf[11];

    channels = NULL;
    ignores = NULL;
    nocaps = NULL;
    intalk = TRUE;
    sputs("welcome to yalk : /? = help : /q = quit\n\n");
    getchannels();
    sputs("\nWhat channel would you like to join? [yabbs] ");
    sgets(buf, 10);
    if (*buf == 0) strcpy(buf, "yabbs");
    sprintf(tmsg, "/j %s", buf);
    dojoin(tmsg);
    do {
        sgets(tmsg, 200); 
        /* move up one line, clear it, and return the to front of it */
        t_puts(t_upcur); sputs("\r"); t_puts(t_cleol); sputs("\r");
        if (tmsg[0] == '/') {
            switch (lower(tmsg[1])) {
                case 'c':   
                    switch (lower(tmsg[2])) {
                        case 'l'    : clrscr(); break;
                        default     : getchannels();
                    }
                    break;
                case 'e':   
                    if (elite) sputs("*** lam3r!\n");
                    else sputs(
                        "*** w3lc0me t0 th3 rank$ 0f th3 |<-rad 3l33t, d00d!\n"
                    );
                    elite = !elite; 
                    break;
                case 'h':   bbs_talkhide(curchan); break;
                case 'i':
                    switch (lower(tmsg[2])) {
                        case 'g'    : doignore(tmsg); break;
                        default     : doinfo(tmsg); 
                    }
                    break;
                case 'j':   dojoin(tmsg); break;
                case 'l':   doleave(tmsg); break;
                case 'm':   domsg(tmsg); break; 
                case 'p':   dopage(tmsg); break;
                case 'q':   done = TRUE; break;
                case 's':   bbs_talkshow(curchan); break;
                case 'w':   who(); break;
                case 'v':   donocaps(tmsg); break;
                case 'y':   
                    if (yeck) sputs("*** dude, you suck!\n");
                    else sputs("*** welcome to yeck mode, dude!\n");
                    yeck = !yeck; 
                    break;
                case '?':   sputs_mime(H_TALK); break;
                default :   sputs("*** unknown command (hit /? for help)\n");
            }
        } else if ((tmsg[0] == ':') && (strlen(tmsg) > 3)) {
            doaction(tmsg);
        } else if (strlen(tmsg) != 0) {
            dowrite(tmsg);  
        }
    } while (!done);

    intalk = FALSE;
    while (channels != NULL) {
        bbs_talkleave((char *) channels->ptr);
        channels = deletefromlist(channels, channels->ptr, stringptrcomp);
    }
}
